@@ -1,11 +1,10 @@
 """
 API for NumPy usage.
 """
-
 import functools
 from typing import Any, Callable, List, Optional, Sequence, Tuple
 
-import numpy as np  # type: ignore
+import numpy as np
 
 from targetran._check import (
     _check_shear_input,
@@ -42,14 +41,16 @@ from targetran._transform import (
     _get_shear_mats,
     _get_translate_mats,
 )
+from targetran._typing import NDFloatArray, NDIntArray
 from targetran.utils import Interpolation
 
 
 def _np_get_affine_dependency() -> _AffineDependency:
     return _AffineDependency(
         _np_convert, np.shape, np.reshape, np.expand_dims, np.squeeze,
-        _np_pad_image, _np_range, _np_cast_to_int, _np_round_to_int,
-        np.repeat, np.tile, np.ones_like, np.stack, np.concatenate, np.matmul,
+        _np_pad_image, _np_range, _np_cast_to_int, _np_round_to_int, np.repeat,
+        np.tile,  # type: ignore
+        np.ones_like, np.stack, np.concatenate, np.matmul,
         np.clip, np.floor, np.ceil, _np_gather_image,
         np.copy, np.max, np.min,
         _np_logical_and, _np_boolean_mask
@@ -57,13 +58,13 @@ def _np_get_affine_dependency() -> _AffineDependency:
 
 
 def _np_affine_transform(
-        image: np.ndarray,
-        bboxes: np.ndarray,
-        labels: np.ndarray,
-        image_dest_tran_mat: np.ndarray,
-        bboxes_tran_mat: np.ndarray,
+        image: NDFloatArray,
+        bboxes: NDFloatArray,
+        labels: NDFloatArray,
+        image_dest_tran_mat: NDFloatArray,
+        bboxes_tran_mat: NDFloatArray,
         interpolation: Interpolation
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
     return _affine_transform(
         image, bboxes, labels, image_dest_tran_mat, bboxes_tran_mat,
         interpolation, _np_get_affine_dependency()
@@ -71,10 +72,10 @@ def _np_affine_transform(
 
 
 def flip_left_right(
-        image: np.ndarray,
-        bboxes: np.ndarray,
-        labels: np.ndarray
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+        image: NDFloatArray,
+        bboxes: NDFloatArray,
+        labels: NDFloatArray
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
     return _flip_left_right(
         image, bboxes, labels,
         Interpolation.NEAREST, _np_get_affine_dependency()
@@ -82,10 +83,10 @@ def flip_left_right(
 
 
 def flip_up_down(
-        image: np.ndarray,
-        bboxes: np.ndarray,
-        labels: np.ndarray
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+        image: NDFloatArray,
+        bboxes: NDFloatArray,
+        labels: NDFloatArray
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
     return _flip_up_down(
         image, bboxes, labels,
         Interpolation.NEAREST, _np_get_affine_dependency()
@@ -93,12 +94,12 @@ def flip_up_down(
 
 
 def rotate(
-        image: np.ndarray,
-        bboxes: np.ndarray,
-        labels: np.ndarray,
+        image: NDFloatArray,
+        bboxes: NDFloatArray,
+        labels: NDFloatArray,
         angle_deg: float,
         interpolation: Interpolation = Interpolation.BILINEAR
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
     return _rotate(
         image, bboxes, labels, _np_convert(angle_deg), np.cos, np.sin,
         interpolation, _np_get_affine_dependency()
@@ -106,13 +107,13 @@ def rotate(
 
 
 def shear(
-        image: np.ndarray,
-        bboxes: np.ndarray,
-        labels: np.ndarray,
+        image: NDFloatArray,
+        bboxes: NDFloatArray,
+        labels: NDFloatArray,
         angle_deg: float,
         interpolation: Interpolation = Interpolation.BILINEAR,
         _check_input: bool = True
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
     if _check_input:
         _check_shear_input(angle_deg)
     return _shear(
@@ -122,14 +123,14 @@ def shear(
 
 
 def translate(
-        image: np.ndarray,
-        bboxes: np.ndarray,
-        labels: np.ndarray,
+        image: NDFloatArray,
+        bboxes: NDFloatArray,
+        labels: NDFloatArray,
         translate_height: int,
         translate_width: int,
         interpolation: Interpolation = Interpolation.BILINEAR,
         _check_input: bool = True
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
     if _check_input:
         _check_translate_input(image.shape, translate_height, translate_width)
     return _translate(
@@ -144,8 +145,8 @@ def _np_get_crop_inputs(
         image_width: int,
         height_fraction_range: Tuple[float, float],
         width_fraction_range: Tuple[float, float],
-        rand_fn: Callable[..., np.ndarray]
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
+        rand_fn: Callable[..., NDFloatArray]
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray, NDFloatArray]:
     return _get_crop_inputs(
         image_height, image_width, height_fraction_range, width_fraction_range,
         rand_fn, _np_convert, _np_round_to_int
@@ -153,15 +154,15 @@ def _np_get_crop_inputs(
 
 
 def crop(
-        image: np.ndarray,
-        bboxes: np.ndarray,
-        labels: np.ndarray,
+        image: NDFloatArray,
+        bboxes: NDFloatArray,
+        labels: NDFloatArray,
         offset_height: int,
         offset_width: int,
         crop_height: int,
         crop_width: int,
         _check_input: bool = True
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
     if _check_input:
         _check_crop_input(image.shape, offset_height, offset_width)
     return _crop(
@@ -174,11 +175,11 @@ def crop(
 
 
 def resize(
-        image: np.ndarray,
-        bboxes: np.ndarray,
-        labels: np.ndarray,
+        image: NDFloatArray,
+        bboxes: NDFloatArray,
+        labels: NDFloatArray,
         dest_size: Tuple[int, int]
-) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
     return _resize(
         image, bboxes, labels, dest_size,
         _np_convert, np.shape, np.reshape, _np_resize_image, np.concatenate
@@ -189,7 +190,7 @@ class RandomTransform:
 
     def __init__(
             self,
-            np_fn: Callable[..., Tuple[np.ndarray, np.ndarray, np.ndarray]],
+            np_fn: Callable[..., Tuple[NDFloatArray, NDFloatArray, NDFloatArray]],
             probability: float,
             seed: Optional[int],
             name: str,
@@ -201,24 +202,24 @@ class RandomTransform:
         self.name = name
         self.is_affine = is_affine
 
-    def _rand_fn(self, shape: Tuple[int, ...] = ()) -> np.ndarray:
+    def _rand_fn(self, shape: Sequence[int] = ()) -> NDFloatArray:
         return self._rng.random(shape)
 
     def _get_mats(
             self,
-            image: np.ndarray,
-            rand_fn: Callable[..., np.ndarray]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> Tuple[NDFloatArray, NDFloatArray]:
         pass
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray,
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray,
             *args: Any,
             **kwargs: Any
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
         # Make sure inputs are in the needed format.
         image = _np_convert(image)
         bboxes = _np_convert(bboxes)
@@ -263,11 +264,11 @@ class CombineAffine(RandomTransform):
             [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]
         ]), axis=0)
 
-    def _combine_mats(
+    def _get_mats(
             self,
-            image: np.ndarray,
-            rand_fn: Callable[..., np.ndarray]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> Tuple[NDFloatArray, NDFloatArray]:
         image_dest_tran_mats, bboxes_tran_mats, probs = tuple(zip(
             *[(*t._get_mats(image, rand_fn), t.probability)
               for i, t in enumerate(self._transforms)]
@@ -283,10 +284,10 @@ class CombineAffine(RandomTransform):
             bboxes_tran_mats = np.take(bboxes_tran_mats, indices, 0)
         else:
             conditions = np.reshape(rand_fn() < probs, (len(probs), 1, 1))
-            image_dest_tran_mats = np.where(
+            image_dest_tran_mats = np.where(  # type: ignore
                 conditions, image_dest_tran_mats, self._identity_mat
             )
-            bboxes_tran_mats = np.where(
+            bboxes_tran_mats = np.where(  # type: ignore
                 conditions, bboxes_tran_mats, self._identity_mat
             )
 
@@ -301,13 +302,13 @@ class CombineAffine(RandomTransform):
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray,
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray,
             *args: Any,
             **kwargs: Any
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
-        image_dest_tran_mat, bboxes_tran_mat = self._combine_mats(
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
+        image_dest_tran_mat, bboxes_tran_mat = self._get_mats(
             image, self._rand_fn
         )
         return super().__call__(
@@ -329,19 +330,19 @@ class RandomFlipLeftRight(RandomTransform):
 
     def _get_mats(
             self,
-            image: np.ndarray,
-            rand_fn: Callable[..., np.ndarray]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> Tuple[NDFloatArray, NDFloatArray]:
         return _get_flip_left_right_mats(_np_convert)
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray,
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray,
             *args: Any,
             **kwargs: Any
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
         return super().__call__(image, bboxes, labels)
 
 
@@ -358,19 +359,19 @@ class RandomFlipUpDown(RandomTransform):
 
     def _get_mats(
             self,
-            image: np.ndarray,
-            rand_fn: Callable[..., np.ndarray]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> Tuple[NDFloatArray, NDFloatArray]:
         return _get_flip_up_down_mats(_np_convert)
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray,
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray,
             *args: Any,
             **kwargs: Any
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
         return super().__call__(image, bboxes, labels)
 
 
@@ -385,30 +386,35 @@ class RandomRotate(RandomTransform):
     ) -> None:
         _check_input_range(angle_deg_range, None, "angle_deg_range")
         super().__init__(rotate, probability, seed, "RandomRotate", True)
-        self.angle_deg_range = angle_deg_range
+        self.angle_deg_range: NDFloatArray = np.array(angle_deg_range)
         self.interpolation = interpolation
 
-    def _get_angle_deg(self, rand_fn: Callable[..., np.ndarray]) -> np.ndarray:
-        return self.angle_deg_range[1] - self.angle_deg_range[0] \
-               * rand_fn() + self.angle_deg_range[0]
+    def _get_angle_deg(
+            self,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> NDFloatArray:
+        return (  # type: ignore
+            self.angle_deg_range[1] - self.angle_deg_range[0]
+            * rand_fn() + self.angle_deg_range[0]
+        )
 
     def _get_mats(
             self,
-            image: np.ndarray,
-            rand_fn: Callable[..., np.ndarray]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> Tuple[NDFloatArray, NDFloatArray]:
         return _get_rotate_mats(
             self._get_angle_deg(rand_fn), _np_convert, np.cos, np.sin
         )
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray,
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray,
             *args: Any,
             **kwargs: Any
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
         return super().__call__(
             image, bboxes, labels, self._get_angle_deg(self._rand_fn),
             self.interpolation
@@ -426,28 +432,33 @@ class RandomShear(RandomTransform):
     ) -> None:
         _check_input_range(angle_deg_range, (-90.0, 90.0), "angle_deg_range")
         super().__init__(shear, probability, seed, "RandomShear", True)
-        self.angle_deg_range = angle_deg_range
+        self.angle_deg_range: NDFloatArray = np.array(angle_deg_range)
         self.interpolation = interpolation
 
-    def _get_angle_deg(self, rand_fn: Callable[..., np.ndarray]) -> np.ndarray:
-        return self.angle_deg_range[1] - self.angle_deg_range[0] \
-               * rand_fn() + self.angle_deg_range[0]
+    def _get_angle_deg(
+            self,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> NDFloatArray:
+        return (  # type: ignore
+            self.angle_deg_range[1] - self.angle_deg_range[0]
+            * rand_fn() + self.angle_deg_range[0]
+        )
 
     def _get_mats(
             self,
-            image: np.ndarray,
-            rand_fn: Callable[..., np.ndarray]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> Tuple[NDFloatArray, NDFloatArray]:
         return _get_shear_mats(self._get_angle_deg(rand_fn), _np_convert, np.tan)
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray,
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray,
             *args: Any,
             **kwargs: Any
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
         return super().__call__(
             image, bboxes, labels, self._get_angle_deg(self._rand_fn),
             self.interpolation, False
@@ -479,9 +490,9 @@ class RandomTranslate(RandomTransform):
 
     def _get_translate_height_and_width(
             self,
-            image: np.ndarray,
-            rand_fn: Callable[..., np.ndarray]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> Tuple[NDIntArray, NDIntArray]:
         height_fraction, width_fraction = _get_random_size_fractions(
             self.translate_height_fraction_range,
             self.translate_width_fraction_range,
@@ -497,9 +508,9 @@ class RandomTranslate(RandomTransform):
 
     def _get_mats(
             self,
-            image: np.ndarray,
-            rand_fn: Callable[..., np.ndarray]
-    ) -> Tuple[np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            rand_fn: Callable[..., NDFloatArray]
+    ) -> Tuple[NDFloatArray, NDFloatArray]:
         translate_height, translate_width = \
             self._get_translate_height_and_width(image, rand_fn)
         return _get_translate_mats(
@@ -508,12 +519,12 @@ class RandomTranslate(RandomTransform):
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray,
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray,
             *args: Any,
             **kwargs: Any
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
         translate_height, translate_width = \
             self._get_translate_height_and_width(image, self._rand_fn)
         return super().__call__(
@@ -543,12 +554,12 @@ class RandomCrop(RandomTransform):
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray,
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray,
             *args: Any,
             **kwargs: Any
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
         offset_height, offset_width, crop_height, crop_width = \
             _np_get_crop_inputs(
                 np.shape(image)[0], np.shape(image)[1],
@@ -572,8 +583,8 @@ class Resize:
 
     def __call__(
             self,
-            image: np.ndarray,
-            bboxes: np.ndarray,
-            labels: np.ndarray
-    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+            image: NDFloatArray,
+            bboxes: NDFloatArray,
+            labels: NDFloatArray
+    ) -> Tuple[NDFloatArray, NDFloatArray, NDFloatArray]:
         return resize(image, bboxes, labels, self.dest_size)
