@@ -1,12 +1,10 @@
 """
 API for TensorFlow usage.
 """
-
 import functools
 import itertools
-from typing import Any, Callable, List, Optional, Sequence, Tuple, TypeVar
+from typing import Any, Callable, List, Optional, Sequence, Tuple
 
-import numpy as np  # type: ignore
 import tensorflow as tf  # type: ignore
 
 from targetran._check import (
@@ -41,10 +39,9 @@ from targetran._transform import (
     _get_shear_mats,
     _get_translate_mats,
 )
+from targetran._typing import T
 from targetran.utils import Interpolation
 
-T = TypeVar("T", np.ndarray, tf.Tensor)
-
 
 def to_tf(
         image_seq: Sequence[T],
@@ -256,7 +253,7 @@ class TFRandomTransform:
         self.name = name
         self.is_affine = is_affine
 
-    def _rand_fn(self, shape: Tuple[int, ...] = ()) -> tf.Tensor:
+    def _rand_fn(self, shape: Sequence[int] = ()) -> tf.Tensor:
         return self._rng.uniform(shape=shape)
 
     def _get_mats(
@@ -300,7 +297,9 @@ def _get_random_indices(
     probs = selected_probabilities if selected_probabilities \
         else tf.ones(num_indices)
     logits = tf.math.log(probs)
-    z = -tf.math.log(-tf.math.log(rng.uniform(tf.shape(logits), 0, 1)))
+    z = -tf.math.log(-tf.math.log(  # pylint: disable=invalid-unary-operand-type
+        rng.uniform(tf.shape(logits), 0, 1)
+    ))
     _, indices = tf.nn.top_k(logits + z, num_selected_indices)
     return indices
 
@@ -339,7 +338,7 @@ class TFCombineAffine(TFRandomTransform):
             [1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]
         ]), axis=0)
 
-    def _combine_mats(
+    def _get_mats(
             self,
             image: tf.Tensor,
             rand_fn: Callable[..., tf.Tensor]
@@ -356,8 +355,12 @@ class TFCombineAffine(TFRandomTransform):
                 self._num_selected_transforms,
                 self._selected_probabilities
             )
-            image_dest_tran_mats = tf.gather(image_dest_tran_mats, indices)
-            bboxes_tran_mats = tf.gather(bboxes_tran_mats, indices)
+            image_dest_tran_mats = tf.gather(  # pylint: disable=no-value-for-parameter
+                image_dest_tran_mats, indices
+            )
+            bboxes_tran_mats = tf.gather(  # pylint: disable=no-value-for-parameter
+                bboxes_tran_mats, indices
+            )
         else:
             conditions = tf.reshape(rand_fn() < probs, (len(probs), 1, 1))
             image_dest_tran_mats = tf.where(
@@ -384,7 +387,7 @@ class TFCombineAffine(TFRandomTransform):
             *args: Any,
             **kwargs: Any
     ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
-        image_dest_tran_mat, bboxes_tran_mat = self._combine_mats(
+        image_dest_tran_mat, bboxes_tran_mat = self._get_mats(
             image, self._rand_fn
         )
         return super().__call__(
