@@ -1,11 +1,10 @@
 """
 Utilities.
 """
-
 from enum import auto, Enum
-from typing import Any, Callable, Sequence, Tuple, TypeVar
+from typing import Any, Callable, Sequence, Tuple
 
-T = TypeVar("T")
+from targetran._typing import T
 
 
 class Interpolation(Enum):
@@ -36,6 +35,6 @@ def image_only(
         transformed_image = tran_fn(image, [], [])[0]
         if len(args) == 0:
             return transformed_image
-        return (transformed_image, *args)
+        return (transformed_image, *args)  # The parentheses are needed.
 
     return fn
